/*!
* MpKit v1.0.9
* (c) 2020-2020 imingyu<mingyuhisoft@163.com>
* Released under the MIT License.
* Github: https://github.com/imingyu/mpkit
*/
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var parse5 = require('parse5');
var types = require('@mpkit/types');
var util = require('@mpkit/util');

var TAG_NOT_CLOSE = "标签未闭合";
var TAG_HAS_SPACE = "标签开始/结束处存在空白字符";
var BRACKET_THAN_TWO = "括号表达式语法错误：含有多个“{”";
var BRACKET_NOT_CLOSE = "括号表达式语法错误：未闭合，缺少“}}”";
var ATTR_WHERE_NOT_IF = "条件表达式语法错误：缺少“if”语句";
var ATTR_CONTENT_HAS_MORE_VAR = "属性值中的存在多个括号表达式";
var ATTR_KEY_CONTENT_HAS_BRAKET = "key属性值中的存在括号表达式";
var ATTR_FOR_ITEM_NOT_FOR = "for-item属性仅应与for属性一同存在，此时缺少for属性";
var ATTR_FOR_INDEX_NOT_FOR = "for-index属性仅应与for属性一同存在，此时缺少for属性";
var ATTR_FOR_NOT_LISTVAR = "for属性值无可遍历变量或语法错误";
var ATTR_FOR_VAR_EQUAL = "for语法中的list/item/index中存在重复的变量名";
var ATTR_FOR_EACH_VAR_WRONG = "for属性值语法错误，存在“in”却无可遍历变量";

var throwError = (function (data) {
  var err = new Error(data.message);
  util.merge(err, data);
  throw err;
});

var hasAttr = function hasAttr(element, attrName) {
  return element.attrs && element.attrs.some(function (attr) {
    return attr.name === attrName;
  });
};
var parseContent = function parseContent(content, position, target) {
  var result = [];
  var text = "";

  for (var i = 0, len = content.length; i < len; i++) {
    var _char = content[i];

    if (_char === "{") {
      var nextCount = util.nextCharCount(_char, i, content);

      if (nextCount > 1) {
        // 存在"{{{"语法错误
        return throwError({
          message: BRACKET_THAN_TWO,
          target: target,
          position: position
        });
      } else if (nextCount < 1) {
        // 不构成表达式，直接或略
        text += content.substr(i, nextCount + 1);
        i += nextCount;
      } else {
        // 寻找与之对应的"}}"
        var bracketRightIndex = util.firstAfterCharsIndex(i + 2, "}}", content);

        if (bracketRightIndex === -1) {
          // 未找到"}}"
          return throwError({
            message: BRACKET_NOT_CLOSE,
            target: target,
            position: position
          });
        }

        var bracketContent = content.substring(i + 2, bracketRightIndex + 1);

        if (text) {
          result.push({
            type: types.MpXmlContentType["static"],
            value: text
          });
          text = "";
        }

        result.push({
          type: types.MpXmlContentType.dynamic,
          value: bracketContent
        });
        i = bracketRightIndex + 2;
      }
    } else {
      text += _char;
    }
  }

  if (text) {
    result.push({
      type: types.MpXmlContentType["static"],
      value: text
    });
    text = "";
  }

  if (!result.length) {
    result.push({
      type: types.MpXmlContentType["static"],
      value: ""
    });
  }

  return result;
};
var getXmlFrament = function getXmlFrament(xmlRows, startLine, framentOffset) {
  if (framentOffset === void 0) {
    framentOffset = 2;
  }

  startLine = startLine - 1;
  return ">>> " + xmlRows.slice(startLine, startLine + framentOffset > xmlRows.length ? xmlRows.length : startLine + framentOffset).join("\n");
};

var getElementType = function getElementType(node) {
  if (node.nodeName === "#text") {
    return types.MkXmlElementType.text;
  }

  if (node.nodeName === "#comment") {
    return types.MkXmlElementType.comment;
  }

  return types.MkXmlElementType.node;
};

var validateSelfCloseing = function validateSelfCloseing(xml, xmlRows, node, nextNode, element) {
  var _a = node.sourceCodeLocation.startTag,
      startOffset = _a.startOffset,
      endOffset = _a.endOffset,
      startLine = _a.startLine,
      startCol = _a.startCol;

  if (node.tagName === "image") {
    if (nextNode && nextNode.sourceCodeLocation.startOffset > node.sourceCodeLocation.endOffset) {
      return;
    }
  }

  var tagHead = xml.substring(startOffset, endOffset);

  if (tagHead.endsWith("/>")) {
    return;
  }

  return {
    message: TAG_NOT_CLOSE,
    position: types.MkValidateMessagePosition.tag,
    target: element,
    fragment: getXmlFrament(xmlRows, startLine)
  };
}; // const correctTagHead = (orgTagHead: string): string => {
//     let res = "";
//     let emptyChars = "";
//     for (let i = 1, len = orgTagHead.length; i < len; i++) {
//         if (orgTagHead[i].trim()) {
//             break;
//         }
//         emptyChars += orgTagHead[i];
//     }
//     res = orgTagHead.replace(`<${emptyChars}`, "<");
//     emptyChars = "";
//     if (orgTagHead[orgTagHead.length - 1] === ">") {
//         for (let i = orgTagHead.length - 2; i >= 0; i--) {
//             if (orgTagHead[i].trim()) {
//                 break;
//             }
//             emptyChars += orgTagHead[i];
//         }
//         if (emptyChars) {
//             res = res.replace(`${emptyChars}>`, ">");
//         }
//     }
//     return res;
// };


var eachChildren = function eachChildren(xml, xmlRows, children, elements, parseResult) {
  children.forEach(function (childNode, nodeIndex) {
    var _a = childNode.sourceCodeLocation,
        startTag = _a.startTag,
        endTag = _a.endTag,
        startLine = _a.startLine,
        startCol = _a.startCol,
        startOffset = _a.startOffset,
        endOffset = _a.endOffset;
    var xmlElement = {
      type: getElementType(childNode),
      sourceLocationInfo: childNode.sourceCodeLocation
    };

    if (xmlElement.type === types.MkXmlElementType.text) {
      if (childNode.value.indexOf("< ") !== -1 || childNode.value.indexOf(" >") !== -1) {
        // const orgTagHead = childNode.value.trim();
        // let tagHead = correctTagHead(orgTagHead);
        // parseResult.correctXML = xml.replace(orgTagHead, tagHead);
        return throwError({
          message: TAG_HAS_SPACE,
          position: types.MkValidateMessagePosition.tag,
          target: xmlElement,
          fragment: getXmlFrament(xmlRows, startLine)
        });
      }

      xmlElement.content = childNode.value;
      elements.push(xmlElement);
      return;
    }

    if (xmlElement.type === types.MkXmlElementType.comment) {
      xmlElement.content = childNode.data;
      elements.push(xmlElement);
      return;
    }

    if (xmlElement.type === types.MkXmlElementType.node) {
      if (childNode.tagName === "img") {
        var nodeXML = xml.substring(startOffset, endOffset);

        if (nodeXML.indexOf("<image") === 0) {
          childNode.tagName = "image";
        }
      }

      xmlElement.tag = childNode.tagName;

      if (childNode.attrs && childNode.attrs.length) {
        xmlElement.attrs = [];
        childNode.attrs.forEach(function (attr) {
          var attrData = {
            name: attr.name,
            content: attr.value,
            sourceLocationInfo: childNode.sourceCodeLocation.attrs[attr.name]
          };
          var attrXml = xml.substring(childNode.sourceCodeLocation.attrs[attr.name].startOffset, childNode.sourceCodeLocation.attrs[attr.name].endOffset);

          if (attrXml === attr.name) {
            delete attrData.content;
          }

          xmlElement.attrs.push(attrData);
        });
      }

      if (startTag && !endTag) {
        var res = validateSelfCloseing(xml, xmlRows, childNode, children[nodeIndex + 1], xmlElement);

        if (res) {
          return throwError(res);
        }

        if (xmlElement.tag === "import" || xmlElement.tag === "include") {
          util.merge(xmlElement.sourceLocationInfo, xmlElement.sourceLocationInfo.startTag);
        }

        xmlElement.selfCloseing = true;
        elements.push(xmlElement);
        return eachChildren(xml, xmlRows, childNode.childNodes, elements);
      } else if (childNode.childNodes && childNode.childNodes.length) {
        xmlElement.children = [];
        elements.push(xmlElement);
        return eachChildren(xml, xmlRows, childNode.childNodes, xmlElement.children);
      } else {
        return elements.push(xmlElement);
      }
    }
  });
};

var parseXML = function parseXML(xml) {
  xml = xml.trim();
  var result = {
    elements: [],
    xml: xml,
    xmlRows: xml.split("\n")
  };
  var fragmentJSON = parse5.parseFragment(xml, {
    sourceCodeLocationInfo: true
  });

  try {
    eachChildren(xml, result.xmlRows, fragmentJSON.childNodes, result.elements, result);
  } catch (error) {
    result.error = error;
  }

  return result;
};

function _typeof(obj) {
  "@babel/helpers - typeof";

  if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {
    _typeof = function (obj) {
      return typeof obj;
    };
  } else {
    _typeof = function (obj) {
      return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    };
  }

  return _typeof(obj);
}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var ParseElementAdapterImpl =
/** @class */
function () {
  function ParseElementAdapterImpl(attrAdapters, contentAdapter) {
    this.attrAdapters = attrAdapters;
    this.contentAdapter = contentAdapter;
  }

  ParseElementAdapterImpl.prototype.parse = function (data) {
    var _this = this;

    var currentElement = data.currentElement,
        currentElementIndex = data.currentElementIndex,
        brotherElements = data.brotherElements,
        allElements = data.allElements,
        orgXml = data.orgXml;

    if (currentElement.type === types.MkXmlElementType.node) {
      var attrs = void 0;

      if (currentElement.attrs) {
        var commonAttr_1 = this.attrAdapters["*"];
        var unclaimedAttr_1 = this.attrAdapters.unclaimed;
        attrs = currentElement.attrs.map(function (item, index, arr) {
          var resultAttr;
          var currentAttr = _this.attrAdapters[item.name];

          if (currentAttr) {
            resultAttr = currentAttr.parse({
              currentElement: currentElement,
              currentElementIndex: currentElementIndex,
              brotherElements: brotherElements,
              allElements: allElements,
              orgXml: orgXml,
              currentAttr: item,
              allAttrs: arr
            });
          } else if (unclaimedAttr_1) {
            resultAttr = unclaimedAttr_1.parse({
              currentElement: currentElement,
              currentElementIndex: currentElementIndex,
              brotherElements: brotherElements,
              allElements: allElements,
              orgXml: orgXml,
              currentAttr: item,
              allAttrs: arr,
              prevParseAttr: resultAttr
            });
          }

          if (commonAttr_1) {
            resultAttr = commonAttr_1.parse({
              currentElement: currentElement,
              currentElementIndex: currentElementIndex,
              brotherElements: brotherElements,
              allElements: allElements,
              orgXml: orgXml,
              currentAttr: item,
              allAttrs: arr,
              prevParseAttr: resultAttr
            });
          }

          if (!resultAttr) {
            resultAttr = item;

            if (item.content && item.content.trim()) {
              try {
                resultAttr.content = _this.contentAdapter.parse(item.content);
              } catch (error) {
                var err = error;
                err.position = types.MkValidateMessagePosition.attr;
                err.target = resultAttr;
                throw err;
              }
            }
          }

          return resultAttr;
        });
      }

      var result = currentElement;

      if (attrs) {
        result.attrs = attrs;
      }

      return result;
    }

    if (currentElement.type === types.MkXmlElementType.text && currentElement.content && currentElement.content.trim()) {
      var result = currentElement;

      try {
        var content = this.contentAdapter.parse(currentElement.content);
        result.content = content;
      } catch (error) {
        var err = error;
        err.position = types.MkValidateMessagePosition.content;
        err.target = result;
        throw err;
      }

      return result;
    }

    return currentElement;
  };

  return ParseElementAdapterImpl;
}();

var MpParseElementAdapter =
/** @class */
function (_super) {
  __extends(MpParseElementAdapter, _super);

  function MpParseElementAdapter(mpPlatform, attrAdapters, contentAdapter) {
    var _this = _super.call(this, attrAdapters, contentAdapter) || this;

    _this.mpPlatform = mpPlatform;
    return _this;
  }

  return MpParseElementAdapter;
}(ParseElementAdapterImpl);

var ParseContentAdapterImpl =
/** @class */
function () {
  function ParseContentAdapterImpl() {}

  ParseContentAdapterImpl.prototype.parse = function (content) {
    return parseContent(content);
  };

  return ParseContentAdapterImpl;
}();

var MpParseContentAdapter =
/** @class */
function (_super) {
  __extends(MpParseContentAdapter, _super);

  function MpParseContentAdapter(mpPlatform) {
    var _this = _super.call(this) || this;

    _this.mpPlatform = mpPlatform;
    return _this;
  }

  return MpParseContentAdapter;
}(ParseContentAdapterImpl);

var _a;
var MpWechatViewSyntaxSpec = {
  namespace: "wx:",
  "for": "for",
  forItem: "for-item",
  forIndex: "for-index",
  key: "key",
  "if": "if",
  elseif: "elif",
  "else": "else",
  forAndWhereAttrNeedBracket: true
};
var MpSpec = {
  ViewSyntax: (_a = {}, _a[types.MpPlatform.wechat] = MpWechatViewSyntaxSpec, _a[types.MpPlatform.alipay] = util.merge({}, MpWechatViewSyntaxSpec, {
    namespace: "a:"
  }), _a[types.MpPlatform.smart] = util.merge({}, MpWechatViewSyntaxSpec, {
    namespace: "s-",
    forAndWhereAttrNeedBracket: false
  }), _a[types.MpPlatform.tiktok] = util.merge({}, MpWechatViewSyntaxSpec, {
    namespace: "tt:"
  }), _a)
};

var MpBaseParseAttrAdapter =
/** @class */
function () {
  function MpBaseParseAttrAdapter(mpPlatform) {
    this.allowMoreContentVar = true;
    this.mpPlatform = mpPlatform;
    var spec = MpSpec.ViewSyntax[this.mpPlatform];
    this.mpViewSyntax = spec;
  }

  MpBaseParseAttrAdapter.prototype.parse = function (data) {
    var attr = data.currentAttr;
    var content = this.parseContent(data);

    if (Array.isArray(content)) {
      attr.content = content;
    }

    return attr;
  };

  MpBaseParseAttrAdapter.prototype.parseContent = function (data) {
    var currentAttr = data.currentAttr;

    if ("content" in currentAttr) {
      if (this.mpViewSyntax.forAndWhereAttrNeedBracket) {
        var contents = parseContent(currentAttr.content, types.MkValidateMessagePosition.attr, currentAttr);

        if (!this.allowMoreContentVar) {
          var filterEmpty = contents.filter(function (item) {
            if (item.type === types.MpXmlContentType.dynamic) {
              return true;
            }

            return item.value && item.value.trim();
          });

          if (filterEmpty.length > 1) {
            return throwError({
              message: ATTR_CONTENT_HAS_MORE_VAR,
              position: types.MkValidateMessagePosition.attr,
              target: currentAttr
            });
          }

          return filterEmpty;
        }

        return contents;
      }

      return [{
        type: types.MpXmlContentType.dynamic,
        value: currentAttr.content
      }];
    }
  };

  return MpBaseParseAttrAdapter;
}();

var MpParseWehreAttrAdapter =
/** @class */
function (_super) {
  __extends(MpParseWehreAttrAdapter, _super);

  function MpParseWehreAttrAdapter(mpPlatform) {
    var _this = _super.call(this, mpPlatform) || this;

    _this.allowMoreContentVar = false;
    var spec = _this.mpViewSyntax;
    _this.mpViewSyntax = spec;
    _this.ifValue = spec.namespace + spec["if"];
    _this.elseifValue = spec.namespace + spec.elseif;
    _this.elseValue = spec.namespace + spec["else"];
    return _this;
  }

  MpParseWehreAttrAdapter.prototype.parse = function (data) {
    var currentAttr = data.currentAttr,
        currentElementIndex = data.currentElementIndex,
        brotherElements = data.brotherElements;
    var attrName = currentAttr.name;

    if (attrName === this.elseifValue) {
      var brotherElement = brotherElements[currentElementIndex - 1];

      if (!brotherElement || !hasAttr(brotherElement, this.ifValue)) {
        return throwError({
          message: ATTR_WHERE_NOT_IF,
          position: types.MkValidateMessagePosition.attr,
          target: currentAttr
        });
      }
    }

    if (attrName === this.elseValue) {
      var brotherElement = brotherElements[currentElementIndex - 1];

      if (!brotherElement || !hasAttr(brotherElement, this.ifValue) && !hasAttr(brotherElement, this.elseifValue)) {
        return throwError({
          message: ATTR_WHERE_NOT_IF,
          position: types.MkValidateMessagePosition.attr,
          target: currentAttr
        });
      }
    }

    var attr = data.currentAttr;
    var content = this.parseContent(data);

    if (Array.isArray(content)) {
      attr.content = content;
    }

    return attr;
  };

  return MpParseWehreAttrAdapter;
}(MpBaseParseAttrAdapter);

var MpParseForAttrAdapter =
/** @class */
function (_super) {
  __extends(MpParseForAttrAdapter, _super);

  function MpParseForAttrAdapter(mpPlatform) {
    var _this = _super.call(this, mpPlatform) || this;

    _this.allowMoreContentVar = false;
    _this.mpPlatform = mpPlatform;
    var spec = _this.mpViewSyntax;
    _this.forValue = spec.namespace + spec["for"];
    _this.forItemValue = spec.namespace + spec.forItem;
    _this.forIndexValue = spec.namespace + spec.forIndex;
    _this.forKeyValue = spec.namespace + spec.key;
    return _this;
  }

  MpParseForAttrAdapter.prototype.mergeForContent = function (content, key, attr) {
    if (attr && attr.content && attr.content.trim()) {
      content[key] = attr.content.trim();
    }
  };

  MpParseForAttrAdapter.prototype.getAttrValue = function (attr, defaultValue) {
    if (defaultValue === void 0) {
      defaultValue = "";
    }

    return attr && attr.content && attr.content.trim() ? attr.content.trim() : defaultValue;
  };

  MpParseForAttrAdapter.prototype.mergeEachVar = function (eachVar, data, forAttrContent, forItemAttr, forIndexAttr, hasIn) {
    var itemVal;
    var indexVal;
    eachVar = eachVar.trim();

    if (eachVar.indexOf(",") !== -1) {
      var arr = this.splitString(eachVar, ",").filter(function (item) {
        return item;
      });

      if (!arr.length && hasIn) {
        return throwError({
          message: ATTR_FOR_EACH_VAR_WRONG,
          position: types.MkValidateMessagePosition.attr,
          target: data.currentAttr
        });
      }

      itemVal = arr[0] || this.getAttrValue(forItemAttr);
      indexVal = arr[1] || this.getAttrValue(forIndexAttr);
    } else if (eachVar) {
      itemVal = hasIn ? eachVar || this.getAttrValue(forItemAttr) : this.getAttrValue(forItemAttr);
      indexVal = this.getAttrValue(forIndexAttr);
    } else if (hasIn) {
      return throwError({
        message: ATTR_FOR_EACH_VAR_WRONG,
        position: types.MkValidateMessagePosition.attr,
        target: data.currentAttr
      });
    }

    if (hasIn && (!itemVal || !indexVal)) {
      return throwError({
        message: ATTR_FOR_EACH_VAR_WRONG,
        position: types.MkValidateMessagePosition.attr,
        target: data.currentAttr
      });
    }

    if (itemVal) {
      forAttrContent.featureItem = itemVal;
    }

    if (indexVal) {
      forAttrContent.featureIndex = indexVal;
    }
  };

  MpParseForAttrAdapter.prototype.splitString = function (str, _char) {
    return str.split(_char).map(function (item) {
      return item.trim();
    });
  };

  MpParseForAttrAdapter.prototype.parseForAttrContent = function (data, forAttrContent) {
    var _this = this;

    var result = forAttrContent;

    if (typeof forAttrContent.value !== "string") {
      return throwError({
        message: ATTR_FOR_NOT_LISTVAR,
        position: types.MkValidateMessagePosition.attr,
        target: data.currentAttr
      });
    }

    var attrVal = forAttrContent.value.trim();

    if (!attrVal) {
      return throwError({
        message: ATTR_FOR_NOT_LISTVAR,
        position: types.MkValidateMessagePosition.attr,
        target: data.currentAttr
      });
    }

    var forItemAttr = data.allAttrs.find(function (item) {
      return item.name === _this.forItemValue;
    });
    var forIndexAttr = data.allAttrs.find(function (item) {
      return item.name === _this.forIndexValue;
    });

    if (this.mpPlatform === types.MpPlatform.wechat || this.mpPlatform === types.MpPlatform.alipay || this.mpPlatform === types.MpPlatform.tiktok) {
      var forKeyAttr = data.allAttrs.find(function (item) {
        return item.name === _this.forKeyValue;
      });
      result.featureList = attrVal;
      this.mergeForContent(result, "featureItem", forItemAttr);
      this.mergeForContent(result, "featureKey", forKeyAttr);
      this.mergeForContent(result, "featureIndex", forIndexAttr);
    } else if (this.mpPlatform === types.MpPlatform.smart) {
      // 百度小程序语法：s-for="item,index in list trackBy index"
      var _a = this.splitString(attrVal, "trackBy"),
          valMain = _a[0],
          valKey = _a[1];

      if (valKey) {
        result.featureKey = valKey;
      }

      if (valMain.indexOf(" in ") !== -1) {
        var _b = this.splitString(valMain, " in "),
            eachVar = _b[0],
            listVar = _b[1];

        if (!listVar) {
          return throwError({
            message: ATTR_FOR_NOT_LISTVAR,
            position: types.MkValidateMessagePosition.attr,
            target: data.currentAttr
          });
        }

        result.featureList = listVar;
        this.mergeEachVar(eachVar, data, result, forItemAttr, forIndexAttr, true);
      } else {
        this.mergeEachVar(valMain, data, result, forItemAttr, forIndexAttr, false);
        result.featureList = valMain;
      }
    }

    return result;
  };

  MpParseForAttrAdapter.prototype.validateForVarName = function (forContent, attr) {
    ["featureItem", "featureIndex", "featureList"].forEach(function (item, index, arr) {
      var val = forContent[item];

      if (val) {
        var equalItem = arr.filter(function (t) {
          return t !== item;
        }).find(function (t) {
          return forContent[t] === val;
        });

        if (equalItem) {
          return throwError({
            message: ATTR_FOR_VAR_EQUAL,
            position: types.MkValidateMessagePosition.attr,
            target: attr
          });
        }
      }
    });
  };

  MpParseForAttrAdapter.prototype.parse = function (data) {
    var currentAttr = data.currentAttr,
        currentElement = data.currentElement;
    var attrName = currentAttr.name;

    if (attrName === this.forItemValue) {
      // for-item一定要与for属性一同出现
      if (!hasAttr(currentElement, this.forValue)) {
        return throwError({
          message: ATTR_FOR_ITEM_NOT_FOR,
          position: types.MkValidateMessagePosition.attr,
          target: currentAttr
        });
      }
    }

    if (attrName === this.forIndexValue) {
      // for-index一定要与for属性一同出现
      if (!hasAttr(currentElement, this.forValue)) {
        return throwError({
          message: ATTR_FOR_INDEX_NOT_FOR,
          position: types.MkValidateMessagePosition.attr,
          target: currentAttr
        });
      }
    }

    var attr = data.currentAttr;
    var contents = this.parseContent(data);

    if (Array.isArray(contents)) {
      if (attrName === this.forValue) {
        var forContent = this.parseForAttrContent(data, contents[0]);
        this.validateForVarName(forContent, attr);
        contents[0] = forContent;
      }

      attr.content = contents;
    }

    if (attrName === this.forKeyValue && this.mpViewSyntax.forAndWhereAttrNeedBracket && contents[0].type === types.MpXmlContentType.dynamic) {
      // key="{{sdf}}"
      return throwError({
        message: ATTR_KEY_CONTENT_HAS_BRAKET,
        position: types.MkValidateMessagePosition.attr,
        target: currentAttr
      });
    }

    return attr;
  };

  return MpParseForAttrAdapter;
}(MpBaseParseAttrAdapter);

var _a$1; // 处理节点

var initElementAdapter = function initElementAdapter(mpPlatform) {
  var _a;

  var contentAdapter = new MpParseContentAdapter(mpPlatform);
  var whereAttrAdapter = new MpParseWehreAttrAdapter(mpPlatform);
  var forAttrAdapter = new MpParseForAttrAdapter(mpPlatform);
  return new MpParseElementAdapter(mpPlatform, (_a = {}, _a[whereAttrAdapter.ifValue] = whereAttrAdapter, _a[whereAttrAdapter.elseifValue] = whereAttrAdapter, _a[whereAttrAdapter.elseValue] = whereAttrAdapter, _a[forAttrAdapter.forValue] = forAttrAdapter, _a[forAttrAdapter.forItemValue] = forAttrAdapter, _a[forAttrAdapter.forIndexValue] = forAttrAdapter, _a[forAttrAdapter.forKeyValue] = forAttrAdapter, _a), contentAdapter);
};

var MpAdapter = (_a$1 = {}, _a$1[types.MpPlatform.wechat] = initElementAdapter(types.MpPlatform.wechat), _a$1[types.MpPlatform.alipay] = initElementAdapter(types.MpPlatform.alipay), _a$1[types.MpPlatform.smart] = initElementAdapter(types.MpPlatform.smart), _a$1[types.MpPlatform.tiktok] = initElementAdapter(types.MpPlatform.tiktok), _a$1);

var parseMpXml = function parseMpXml(mpXml, adapter) {
  var xmlParseResult = parseXML(mpXml);

  if (xmlParseResult.error) {
    delete xmlParseResult.elements;
    return xmlParseResult;
  }

  var parseAdapter;

  if (_typeof(adapter) === "object" && adapter.parse) {
    parseAdapter = adapter;
  } else if (typeof adapter === "string" && types.MpPlatform[adapter]) {
    parseAdapter = MpAdapter[adapter];
  } else {
    throw new Error("adapter参数有误，无法解析");
  }

  var eachParse = function eachParse(elements, allElements, parentElement) {
    return elements.map(function (item, index, arr) {
      var res = parseAdapter.parse({
        currentElement: item,
        currentElementIndex: index,
        brotherElements: arr,
        allElements: allElements,
        orgXml: xmlParseResult.xml
      });

      if (parentElement) {
        res.parent = parentElement;
      }

      var children;

      if (item.children) {
        children = eachParse(item.children, allElements, res);
      }

      if (children && children.length) {
        res.children = children;
      }

      return res;
    });
  };

  try {
    var allElements = xmlParseResult.elements;
    var elements = eachParse(allElements, allElements);
    var result = xmlParseResult;
    result.elements = elements;
    return result;
  } catch (error) {
    var result = xmlParseResult;
    delete result.elements;
    result.error = error;
    return result;
  }
};

var copyValye = function copyValye(options, prop, source, target) {
  if (options[prop]) {
    target[prop] = source[prop];
  }
};

var mpXmlParseResultToJSON = function mpXmlParseResultToJSON(result, options) {
  var json = {};
  options = options || {};
  copyValye(options, "xml", result, json);
  copyValye(options, "xmlRows", result, json);

  if (options.sourceLocationInfo) {
    json.elements = result.elements;
  } else {
    json.elements = result.elements.map(function (item) {
      delete item.sourceLocationInfo;

      if (item.attrs) {
        item.attrs = item.attrs.map(function (attr) {
          delete attr.sourceLocationInfo;
          return attr;
        });
      }

      return item;
    });
  }

  return json;
};

var version = "1.0.9";

exports.ATTR_CONTENT_HAS_MORE_VAR = ATTR_CONTENT_HAS_MORE_VAR;
exports.ATTR_FOR_EACH_VAR_WRONG = ATTR_FOR_EACH_VAR_WRONG;
exports.ATTR_FOR_INDEX_NOT_FOR = ATTR_FOR_INDEX_NOT_FOR;
exports.ATTR_FOR_ITEM_NOT_FOR = ATTR_FOR_ITEM_NOT_FOR;
exports.ATTR_FOR_NOT_LISTVAR = ATTR_FOR_NOT_LISTVAR;
exports.ATTR_FOR_VAR_EQUAL = ATTR_FOR_VAR_EQUAL;
exports.ATTR_KEY_CONTENT_HAS_BRAKET = ATTR_KEY_CONTENT_HAS_BRAKET;
exports.ATTR_WHERE_NOT_IF = ATTR_WHERE_NOT_IF;
exports.BRACKET_NOT_CLOSE = BRACKET_NOT_CLOSE;
exports.BRACKET_THAN_TWO = BRACKET_THAN_TWO;
exports.MpParseContentAdapter = MpParseContentAdapter;
exports.MpParseElementAdapter = MpParseElementAdapter;
exports.ParseContentAdapterImpl = ParseContentAdapterImpl;
exports.ParseElementAdapterImpl = ParseElementAdapterImpl;
exports.TAG_HAS_SPACE = TAG_HAS_SPACE;
exports.TAG_NOT_CLOSE = TAG_NOT_CLOSE;
exports.mpXmlParseResultToJSON = mpXmlParseResultToJSON;
exports.parseMpXml = parseMpXml;
exports.parseXML = parseXML;
exports.version = version;
//# sourceMappingURL=index.cjs.js.map
